@@ -1,5 +1,5 @@
 /* Use of program name in error-reporting functions.
-   Copyright (C) 2001-2003, 2006 Free Software Foundation, Inc.
+   Copyright (C) 2001-2003, 2006, 2019, 2023 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
    This program is free software: you can redistribute it and/or modify
@@ -13,7 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 
 #include <config.h>
@@ -22,8 +22,13 @@
 #include "error-progname.h"
 
 #include <stdio.h>
+#include <stdlib.h>
 
-#include "progname.h"
+#if IN_LIBGETTEXTPO
+# define program_name getprogname ()
+#else
+# include "progname.h"
+#endif
 
 
 /* Indicates whether errors and warnings get prefixed with program_name.
