@@ -1,5 +1,5 @@
 /* Python format strings.
-   Copyright (C) 2001-2004, 2006-2009, 2015-2016 Free Software Foundation, Inc.
+   Copyright (C) 2001-2004, 2006-2009, 2019-2020, 2023 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
    This program is free software: you can redistribute it and/or modify
@@ -13,7 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -92,7 +92,6 @@ struct spec
   unsigned int directives;
   unsigned int named_arg_count;
   unsigned int unnamed_arg_count;
-  unsigned int allocated;
   struct named_arg *named;
   struct unnamed_arg *unnamed;
 };
@@ -120,14 +119,15 @@ format_parse (const char *format, bool translated, char *fdi,
 {
   const char *const format_start = format;
   struct spec spec;
+  unsigned int allocated;
   struct spec *result;
 
   spec.directives = 0;
   spec.named_arg_count = 0;
   spec.unnamed_arg_count = 0;
-  spec.allocated = 0;
   spec.named = NULL;
   spec.unnamed = NULL;
+  allocated = 0;
 
   for (; *format != '\0';)
     if (*format++ == '%')
@@ -191,10 +191,10 @@ format_parse (const char *format, bool translated, char *fdi,
                 goto bad_format;
               }
 
-            if (spec.allocated == spec.unnamed_arg_count)
+            if (allocated == spec.unnamed_arg_count)
               {
-                spec.allocated = 2 * spec.allocated + 1;
-                spec.unnamed = (struct unnamed_arg *) xrealloc (spec.unnamed, spec.allocated * sizeof (struct unnamed_arg));
+                allocated = 2 * allocated + 1;
+                spec.unnamed = (struct unnamed_arg *) xrealloc (spec.unnamed, allocated * sizeof (struct unnamed_arg));
               }
             spec.unnamed[spec.unnamed_arg_count].type = FAT_INTEGER;
             spec.unnamed_arg_count++;
@@ -220,10 +220,10 @@ format_parse (const char *format, bool translated, char *fdi,
                     goto bad_format;
                   }
 
-                if (spec.allocated == spec.unnamed_arg_count)
+                if (allocated == spec.unnamed_arg_count)
                   {
-                    spec.allocated = 2 * spec.allocated + 1;
-                    spec.unnamed = (struct unnamed_arg *) xrealloc (spec.unnamed, spec.allocated * sizeof (struct unnamed_arg));
+                    allocated = 2 * allocated + 1;
+                    spec.unnamed = (struct unnamed_arg *) xrealloc (spec.unnamed, allocated * sizeof (struct unnamed_arg));
                   }
                 spec.unnamed[spec.unnamed_arg_count].type = FAT_INTEGER;
                 spec.unnamed_arg_count++;
@@ -288,10 +288,10 @@ format_parse (const char *format, bool translated, char *fdi,
                 goto bad_format;
               }
 
-            if (spec.allocated == spec.named_arg_count)
+            if (allocated == spec.named_arg_count)
               {
-                spec.allocated = 2 * spec.allocated + 1;
-                spec.named = (struct named_arg *) xrealloc (spec.named, spec.allocated * sizeof (struct named_arg));
+                allocated = 2 * allocated + 1;
+                spec.named = (struct named_arg *) xrealloc (spec.named, allocated * sizeof (struct named_arg));
               }
             spec.named[spec.named_arg_count].name = name;
             spec.named[spec.named_arg_count].type = type;
@@ -309,10 +309,10 @@ format_parse (const char *format, bool translated, char *fdi,
                 goto bad_format;
               }
 
-            if (spec.allocated == spec.unnamed_arg_count)
+            if (allocated == spec.unnamed_arg_count)
               {
-                spec.allocated = 2 * spec.allocated + 1;
-                spec.unnamed = (struct unnamed_arg *) xrealloc (spec.unnamed, spec.allocated * sizeof (struct unnamed_arg));
+                allocated = 2 * allocated + 1;
+                spec.unnamed = (struct unnamed_arg *) xrealloc (spec.unnamed, allocated * sizeof (struct unnamed_arg));
               }
             spec.unnamed[spec.unnamed_arg_count].type = type;
             spec.unnamed_arg_count++;
@@ -446,7 +446,7 @@ format_check (void *msgid_descr, void *msgstr_descr, bool equality,
           unsigned int n1 = spec1->named_arg_count;
           unsigned int n2 = spec2->named_arg_count;
 
-          /* Check the argument names are the same.
+          /* Check that the argument names are the same.
              Both arrays are sorted.  We search for the first difference.  */
           for (i = 0, j = 0; i < n1 || j < n2; )
             {
@@ -688,7 +688,7 @@ main ()
 /*
  * For Emacs M-x compile
  * Local Variables:
- * compile-command: "/bin/sh ../libtool --tag=CC --mode=link gcc -o a.out -static -O -g -Wall -I.. -I../gnulib-lib -I../intl -DHAVE_CONFIG_H -DTEST format-python.c ../gnulib-lib/libgettextlib.la"
+ * compile-command: "/bin/sh ../libtool --tag=CC --mode=link gcc -o a.out -static -O -g -Wall -I.. -I../gnulib-lib -I../../gettext-runtime/intl -DHAVE_CONFIG_H -DTEST format-python.c ../gnulib-lib/libgettextlib.la"
  * End:
  */
 
